/**
 * 
 * CREOFF VAT CALCULATOR ON AIR
 * 
 * @author Jos Deldime | jos.deldime@creoff.net
 * @copyright Jos Deldime - CREOFF 2008
 * 
 * An Air version of the similar Yahoo Widget from CreOFF
 * visit blog.creoff.net for most recent news and updates
 * 
 * -----------------------
 *  CHANGELOG:
 * 
 *	v1.0
 * 	- the first AIR version, based on the original v1.1 Yahoo widget code.
 * 	http://widgets.yahoo.com/widgets/creoff-vat-calculator
 * 
 * -------------------- 
 * */

/*
 * startup engine after all resources have loaded
 */
$(document).ready(function(){

	app.log("document ready");				
	app.init();	

});
	

var app = {

	// default preferences:
	preferences: {
		percentages:  "6,12,21", /* These are the Belgium VAT percentages (yep, we pay 21% !!) */
		defaultPercentage: 21,
		x: 20,
		y: 20
	},
	prefsFileName: "vatcalcprefs.ini",
	aboutpage: {},
	version: 0,
	percentages: [],
	_currPercentage: 0,
	_currInput: "",

	/**
	 * call this to initialize application, load preferences etc etc
	 */
	init: function() {
		this.log("init app");
		
		var xmlString = air.NativeApplication.nativeApplication.applicationDescriptor;
		var appXml = new DOMParser();
		var xmlobject = appXml.parseFromString(xmlString, "text/xml");
		var root = xmlobject.getElementsByTagName('application')[0];
		this.version = root.getElementsByTagName("version")[0].firstChild.data;
		this.log("version:"+ this.version);
		$(".version").html("v"+ this.version );		
		
		this.initPrefs();
		this.initEvents();
	},
	
	/**
	 * helper function (mostly just shortcut to the long air dump statement)
	 * 
	 * @param {string} txt
	 * @return {screenoutput} dump of string/object or nothing if Introspector is not loaded
	 */
	log: function(arg) {
		
		if (air.Introspector) { // log string or object
			air.Introspector.Console.log(typeof(arg));
			if (typeof(arg)=='string') {
				air.Introspector.Console.log(arg);
			} else {
				air.Introspector.Console.dump(arg);
			}							
		} // otherwise be quiet
		
	},

	/***
	* load preferences and initializes objects with the values
	*/
	initPrefs: function() {
		
		// load app.preferences
		this.loadPrefs();

		// position the window
		var screenBounds = air.Screen.mainScreen.visibleBounds;
		app.log(screenBounds);
		
		// adjust positions (for now right and bottom are checked)
		// adjust x if necessary
		if (this.preferences.x + 320 > screenBounds.right ) {
			this.preferences.x = screenBounds.right - 320;
		}
		
		// adjust y if necessary
		if (this.preferences.y + 220 > screenBounds.bottom ) {
			this.preferences.y = screenBounds.bottom - 220;
		}

		// position at x,y 
		window.nativeWindow.x = this.preferences.x;
		window.nativeWindow.y = this.preferences.y;				
					
		// load the percentages into the dropdown		
		var temp = this.preferences.percentages.split(",");
		for (var i=0; i<temp.length; i++) {
			temp[i]= parseFloat(temp[i]); // make sure they're valid numbers
			if (isNaN(temp[i])) temp[i] = 0; // default to zero if contains garbage
			app.log("add percentage "+ temp[i]);
			// is this the selected default?
			var selected = (parseFloat(this.preferences.defaultPercentage) == temp[i]) ? " selected=\"selected\" " : "";
			// add to dropdown if contains valid number
			if (temp[i]>0) $("#selectVATpercentage").append("<option "+selected+" value=\""+temp[i]+"\">"+temp[i]+"%</option>");
		}

	},
	
	/***
	* load preferences
	*/
	loadPrefs: function() {
		
		this.log("loading prefs");		
		
		var file = air.File.applicationStorageDirectory;
		file = file.resolvePath(this.prefsFileName);
		
		if (file.exists) {
			this.log('prefs file exists: ');
			this.log(file);
			
			var fileStream = new air.FileStream();
			fileStream.open(file, air.FileMode.READ);
			this.preferences = fileStream.readObject();
			fileStream.close();	
			
			this.log( app.preferences );
			
		} else {
			app.log('prefs file does not exist');
			app.log(this.preferences);
			
			// create a new prefs file
			var fileStream = new air.FileStream();
			fileStream.open(file, air.FileMode.WRITE);
			fileStream.writeObject(this.preferences);
			fileStream.close();	
		}

	},
	
	/***
	* save preferences
	*/
	savePrefs: function() {		
		this.log("saving prefs");
		
		// add window x,y position
		this.preferences.x = window.nativeWindow.x;
		this.preferences.y = window.nativeWindow.y;
				
		var file = air.File.applicationStorageDirectory;
		file = file.resolvePath(this.prefsFileName);
		try {
			var fileStream = new air.FileStream();
			fileStream.open(file, air.FileMode.UPDATE);
			fileStream.writeObject(this.preferences);
			fileStream.close();		
		} 
		catch(err) {
			app.log("There was an error saving Prefs file: " + err.description );
		}
		

	},
	
	
	/***
	* change preferences
	*/
	changePrefs: function() {		
		this.log("changing prefs");
		this.preferences.percentages = $("#prefvalues").val();		
		$("#selectVATpercentage OPTION").remove();
	},
	
	/***
	* show the preferences 
	*/
	showPrefs: function() {
		
		// Fill input fields with current preferences settings
		$("#prefvalues").val(this.preferences.percentages);
		
		// show DIV holder:
		$("#inputfields").addClass("hidden");
		$("#preferences").removeClass("hidden");		
		
	},
	
	/***
	* hide the preferences 
	*/
	hidePrefs: function() {		
		$("#inputfields").removeClass("hidden");
		$("#preferences").addClass("hidden");		
	},
	
	/*
	* show the about window 
	this is an alternative version with an extra window
	problem was that every time you click on the about button
	a new instance of the window was added
	so it needed more (unnecessary) work to keep track if window was open already
	
	showAbout: function() {
		
		var options = new air.NativeWindowInitOptions();
		options.systemChrome = air.NativeWindowSystemChrome.NONE;
		options.type = air.NativeWindowType.LIGHTWEIGHT;
		options.transparent = true;
		options.resizable = false;
		options.maximizable = false;
		
		var bounds = new air.Rectangle(0, 0, 500, 300);
		this.aboutpage = air.HTMLLoader.createRootWindow(true, options, true, bounds);
		this.aboutpage.load(new air.URLRequest("about.html"));	
		
		//aboutpage.x = 50;
		//aboutpage.y = 30;
		
		//activate and show the new window
		//aboutpage.activate();

	},


	hideAbout: function() {		

		window.nativeWindow.close();

	},*/
	
		
	/***
	* show the about window 
	*/
	showAbout: function() {
		
		// show DIV holder:
		$("#about").removeClass("hidden");
		
		var screenBounds = air.Screen.mainScreen.visibleBounds;
		
		// adjust x if necessary
		if (window.nativeWindow.x + window.nativeWindow.width > screenBounds.right ) {
			window.nativeWindow.x = screenBounds.right - window.nativeWindow.width;
		}
		
		// adjust y if necessary
		if (window.nativeWindow.y + window.nativeWindow.height > screenBounds.bottom ) {
			window.nativeWindow.y = screenBounds.bottom - window.nativeWindow.height;
		}
		
	},
	
	/***
	* hide the about window 
	*/
	hideAbout: function() {		
	
		// show DIV holder:		
		$("#about").addClass("hidden");
		
		// restore position in case you moved it automatically
		window.nativeWindow.x = app.preferences.x;		
		window.nativeWindow.y = app.preferences.y;

	},
	
	/**
	* Calculates the value of a specific field from string to float
	* @param {textobject} inputfieldObj :  the text object to be handled
	* @return {float} 
	*/
   calcField: function(inputfieldObj) {
		var retVal = 0; // default value is 0
		if (inputfieldObj.val()!="") { // evaluate result
			
			var temp = new String(inputfieldObj.val());
			temp = temp.replace(/,/g, "\.") // swap decimal comma for decimal point if present
			
			//remove leading zeros from answer:
			while (temp.charAt(0)=="0") { temp=temp.substring(1,temp.length);} if (temp.charAt(0)==".") temp="0"+temp;
			
			// eval() is not permitted in air (security sandbox)
			// but luckily expressions involving literals are allowed (simple calculations)
			var temp = Number(eval(temp)); 
			if (!isNaN(temp)) retVal = temp; // evaluate this formula
			
		}
		return app.round(retVal);
	},

	/**
	 * Recalculate other fields from currentfield object's value
	 * 
	 */
   recalc: function() {

		// app.log(app._currInput);
		
		_startAmount = app.calcField( $("#txt"+app._currInput) );
		app.log("startamount = "+_startAmount);
		
		_currPercentage = parseFloat( $("#selectVATpercentage").val() )/100;
		app.log("vat % = "+_currPercentage);
		
		switch (app._currInput) {
			case "VATexcl":    // _startAmount = exclusive value
								var tmpIncl = app.round(_startAmount * (1 + _currPercentage));
								var tmpVAT = app.round(_startAmount * _currPercentage);
								$("#txtVATincl").val( tmpIncl );
								$("#txtVATamount").val( tmpVAT );
								break;
								
			case "VATincl":    // inclusive value
								temp =  _startAmount / (1 + _currPercentage); // calc exclusive value
								var tmpexcl = app.round(temp);
								var tmpVAT = app.round(temp * _currPercentage) ;								
								$("#txtVATexcl").val( app.round(temp) );
								$("#txtVATamount").val( app.round(temp * _currPercentage) );
								break;
								
			case "VATamount":   // the VAT value
								var tmpexcl = app.round(_startAmount / _currPercentage);
								var tmpIncl = app.round((_startAmount / _currPercentage) * (1 + _currPercentage));								
								$("#txtVATexcl").val( tmpexcl );
								$("#txtVATincl").val( tmpIncl );								
								break;
		}
	},

	/**
	 * Do this when user moves app window
	 * @return void
	 */
	_onWindowMoved: function() {
		app.log("window has been moved");
		app.savePrefs();
	},
	
	/**
	 * initializes all events in the app (on this screen)
	 */
	initEvents: function() {
				
		// mouseover arrow
		$(".arrow").hover(function(){
			$(this).addClass("hover");
		},function(){
		  	$(this).removeClass("hover");
		});
		
		$(".arrow").click(function(){
			app.setCurrent( $(this).get(0) );
		});
						
		$("#inputfields DIV INPUT").keyup( function(e) {
			
			// if ENTER key hit, just replace field content with calculated content:
			if (e.which==13) {
				$(this).val( app.calcField($(this)) ); // replaces maths like (10*2)+30 with result 50
			}
			
			app.setCurrent( $(this).get(0) );
			app.recalc();
		} );
		
		// 	minimize button		
		$("#selectVATpercentage").change( function() { 
			app.recalc();
			app.preferences.defaultPercentage = $(this).val();
			app.savePrefs();
		} );
		
		// 	minimize app		
		$("#btMinimize").click( function() { 
			app.minimize();
		} );
		
		// 	close app
		$("#btClose").click( function() { 
			app.close();
		} );
		
		// drag app around (zone at top)
		$("#dragzone").mousedown( function() { 
			app.drag();
		} );
		
		// drag app around (zone at top)
		$("#dragzone").mouseup( function() { 
			app._onWindowMoved();
		} );

		// 	save preferences
		$("#btSave").click( function() { 
			app.changePrefs();
			app.savePrefs();
			// reload and re-initialize prefs			
			app.initPrefs();						
			app.hidePrefs();
		} );
		
		$("#btCancel").click( function() { 
			app.hidePrefs();
		} );
		
		// about box
		$("#about").click( function() { 
			app.hideAbout();
		} );

	},
	
	/**
	 * minimizes current app
	 */
	minimize: function() {
		window.nativeWindow.minimize();
	},
	
	/**
	 * allows dragging of current app
	 */
	drag: function() {
		window.nativeWindow.startMove();
	},
	
	/**
	 * closes current app
	 */
	close: function() {
		window.nativeWindow.close();
	},
	
	/**
	 * 
	 */
	setCurrent: function( fieldObj ) {
		// get the parent of the current field
		// (either image or text input)
		var parentObj = fieldObj.parentNode;
		this._currInput = parentObj.id;	
		$("DIV").removeClass('selected');	
		$("#"+this._currInput).addClass('selected');
	},

	round: function(value,precision) {
		if (typeof(precision) == "undefined") precision = 2;
		var temp = Number(value);
		temp = temp.toFixed(precision);
		if (isNaN(temp)) temp = 0;
		return temp;
	}

};
		
			
			